"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Field = void 0;
class Field {
    constructor(form, name) {
        this.setValidators = (validators) => {
            if (this.validators[0] !== validators[0]) {
                this.validators = validators;
                this.validate();
            }
        };
        this.setSubmissionErrors = (errors) => {
            this.errors = errors;
            this.submitFailed = true;
            this.trigger();
        };
        this.setInitialValue = (value) => {
            if (this.initialValue !== value && value !== undefined) {
                this.initialValue = value;
                if (this.value === undefined) {
                    this.value = value;
                    this.inputProps.value = value;
                    this.form.setValue(this.name, value);
                    this.validate();
                    this.trigger();
                }
            }
        };
        this.setDefaultValue = (value) => {
            if (this.defaultValue !== value && value !== undefined) {
                this.defaultValue = value;
                if (this.initialValue === undefined)
                    this.setInitialValue(value);
            }
        };
        this.validate = () => {
            const errors = [];
            for (const validator of this.validators) {
                const result = validator(this.value);
                if (result !== undefined)
                    errors.push(result);
            }
            this.errors = errors;
            return errors.length === 0;
        };
        this.focus = () => {
            this.active = true;
            this.visited = true;
            this.trigger();
        };
        this.change = (value) => {
            this.value = value;
            this.inputProps.value = value !== null && value !== void 0 ? value : "";
            this.form.setValue(this.name, value);
            this.validate();
            this.trigger();
        };
        this.blur = () => {
            this.active = false;
            this.touched = this.visited;
            this.trigger();
        };
        this.reset = () => {
            if (this.value !== this.defaultValue)
                this.change(this.defaultValue);
        };
        this.subscribe = (subscriber) => {
            this.subscribers.add(subscriber);
        };
        this.unsubscribe = (subscriber) => {
            this.subscribers.delete(subscriber);
        };
        this.trigger = () => {
            for (const subscriber of this.subscribers)
                subscriber(this.value);
        };
        this.form = form;
        this.name = name;
        this.validators = [];
        this.errors = [];
        this.initialValue = undefined;
        this.defaultValue = undefined;
        this.value = undefined;
        this.active = false;
        this.visited = false;
        this.touched = false;
        this.submitFailed = false;
        this.inputProps = {
            name: name,
            onChange: (event) => this.change(event.target.value),
            onFocus: () => this.focus(),
            onBlur: () => this.blur(),
            value: "",
        };
        this.subscribers = new Set();
    }
}
exports.Field = Field;
//# sourceMappingURL=field.js.map