"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useForm = exports.useSubscribable = exports.useWrapper = exports.subscribe = exports.Field = exports.Form = exports.DynamicWrapper = exports.StaticWrapper = exports.PromiseExt = void 0;
const React = require("react");
const promise_ext_1 = require("@0x2e757/promise-ext");
Object.defineProperty(exports, "PromiseExt", { enumerable: true, get: function () { return promise_ext_1.PromiseExt; } });
const wrappers_1 = require("@0x2e757/wrappers");
Object.defineProperty(exports, "StaticWrapper", { enumerable: true, get: function () { return wrappers_1.StaticWrapper; } });
Object.defineProperty(exports, "DynamicWrapper", { enumerable: true, get: function () { return wrappers_1.DynamicWrapper; } });
const form_1 = require("./form");
Object.defineProperty(exports, "Form", { enumerable: true, get: function () { return form_1.Form; } });
const field_1 = require("./field");
Object.defineProperty(exports, "Field", { enumerable: true, get: function () { return field_1.Field; } });
const subscribe = (...subscribables) => {
    return (component) => {
        class SubscribedComponent extends component {
            constructor(...args) {
                super(...args);
                const forceUpdateWrapper = () => this.forceUpdate();
                const initialComponentDidMount = this.componentDidMount;
                const initialComponentWillUnmount = this.componentWillUnmount;
                this.componentDidMount = () => {
                    for (const wrapper of subscribables)
                        wrapper.subscribe(forceUpdateWrapper);
                    if (initialComponentDidMount)
                        initialComponentDidMount();
                };
                this.componentWillUnmount = () => {
                    for (const wrapper of subscribables)
                        wrapper.unsubscribe(forceUpdateWrapper);
                    if (initialComponentWillUnmount)
                        initialComponentWillUnmount();
                };
            }
        }
        Object.defineProperty(SubscribedComponent, "name", { value: component.name });
        Object.defineProperty(SubscribedComponent, "displayName", { value: `(Subscribed)(${component.name})` });
        return SubscribedComponent;
    };
};
exports.subscribe = subscribe;
const useWrapper = (wrapper) => {
    const [value, setValue] = React.useState(wrapper.emit());
    React.useEffect(() => {
        wrapper.subscribe(setValue);
        return () => {
            wrapper.unsubscribe(setValue);
        };
    });
    return [value, wrapper.set];
};
exports.useWrapper = useWrapper;
const useSubscribable = (wrapper) => {
    const [value, setValue] = React.useState(wrapper.emit());
    React.useEffect(() => {
        wrapper.subscribe(setValue);
        return () => {
            wrapper.unsubscribe(setValue);
        };
    });
    return value;
};
exports.useSubscribable = useSubscribable;
const useForm = (name) => {
    const [form] = React.useState(new form_1.Form(name));
    return form;
};
exports.useForm = useForm;
exports.default = {
    PromiseExt: promise_ext_1.PromiseExt,
    StaticWrapper: wrappers_1.StaticWrapper,
    DynamicWrapper: wrappers_1.DynamicWrapper,
    Form: form_1.Form,
    Field: field_1.Field,
    subscribe: exports.subscribe,
    useWrapper: exports.useWrapper,
    useSubscribable: exports.useSubscribable,
};
//# sourceMappingURL=index.js.map